import pytest

import pandas.util.testing as tm
import pandas_datareader.data as web

from pandas import DataFrame
from pandas_datareader._utils import RemoteDataError
from pandas_datareader._testing import skip_on_exception
from pandas_datareader.data import DataReader


class TestOptionsWarnings(object):

    def test_options_source_warning(self):
        with tm.assert_produces_warning():
            web.Options('aapl')


class TestDataReader(object):

    @skip_on_exception(RemoteDataError)
    def test_read_yahoo(self):
        gs = DataReader("GS", "yahoo")
        assert isinstance(gs, DataFrame)

    @pytest.mark.xfail(RemoteDataError, reason="failing after #355")
    def test_read_yahoo_dividends(self):
        gs = DataReader("GS", "yahoo-dividends")
        assert isinstance(gs, DataFrame)

    def test_read_google(self):
        gs = DataReader("GS", "google")
        assert isinstance(gs, DataFrame)

    def test_read_fred(self):
        vix = DataReader("VIXCLS", "fred")
        assert isinstance(vix, DataFrame)

    def test_not_implemented(self):
        with pytest.raises(NotImplementedError):
            DataReader("NA", "NA")



import zlib
import os
import time
from pandas.compat import StringIO

import pandas.compat as compat
import pandas as pd
import requests

from pandas_datareader.base import _BaseReader


class EnigmaReader(_BaseReader):
    """
    Collects Enigma data located at the specified datapath and
    returns a pandas DataFrame.

    Usage (high-level):
    ```
        import pandas_datareader as pdr
        df = pdr.get_data_enigma('enigma.inspections.restaurants.fl')

        # in the event that ENIGMA_API_KEY does not exist in your env,
        # it can be supplied as the second arg:
        df = prd.get_data_enigma('enigma.inspections.restaurants.fl',
        ...                      'ARIAMFHKJMISF38UT')
    ```

    Usage:
    ```
        df = EnigmaReader(datapath='enigma.inspections.restaurants.fl',
        ...               api_key='ARIAMFHKJMISF38UT').read()
    ```
    """

    def __init__(self,
                 datapath=None,
                 api_key=None,
                 retry_count=5,
                 pause=0.250,
                 session=None):

        super(EnigmaReader, self).__init__(symbols=[],
                                           retry_count=retry_count,
                                           pause=pause)
        if api_key is None:
            self._api_key = os.getenv('ENIGMA_API_KEY')
            if self._api_key is None:
                raise ValueError("Please provide an Enigma API key or set "
                                 "the ENIGMA_API_KEY environment variable\n"
                                 "If you do not have an API key, you can get "
                                 "one here: https://app.enigma.io/signup")
        else:
            self._api_key = api_key

        self._datapath = datapath
        if not isinstance(self._datapath, compat.string_types):
            raise ValueError(
                "The Enigma datapath must be a string (ex: "
                "'enigma.inspections.restaurants.fl')")

    @property
    def url(self):
        return 'https://api.enigma.io/v2/export/{}/{}'.format(self._api_key,
                                                              self._datapath)

    @property
    def export_key(self):
        return 'export_url'

    @property
    def _head_key(self):
        return 'head_url'

    def _request(self, url):
        self.session.headers.update({'User-Agent': 'pandas-datareader'})
        resp = self.session.get(url)
        resp.raise_for_status()
        return resp

    def _decompress_export(self, compressed_export_data):
        return zlib.decompress(compressed_export_data, 16 + zlib.MAX_WBITS)

    def extract_export_url(self, delay=10, max_attempts=10):
        """
        Performs an HTTP HEAD request on 'head_url' until it returns a `200`.
        This allows the Enigma API time to export the requested data.
        """
        resp = self._request(self.url)
        attempts = 0
        while True:
            try:
                requests.head(resp.json()[self._head_key]).raise_for_status()
            except Exception as e:
                attempts += 1
                if attempts > max_attempts:
                    raise e
                time.sleep(delay)
                continue
            return resp.json()[self.export_key]

    def read(self):
        try:
            return self._read()
        finally:
            self.close()

    def _read(self):
        export_gzipped_req = self._request(self.extract_export_url())
        decompressed_data = self._decompress_export(
            export_gzipped_req.content).decode("utf-8")
        return pd.read_csv(StringIO(decompressed_data))



# pylint: disable-msg=E1101,W0613,W0603

import os

import numpy as np
import pandas as pd
import pandas.util.testing as tm

from pandas_datareader.io import read_jsdmx


class TestJSDMX(object):

    def setup_method(self, method):
        self.dirpath = tm.get_data_path()

    def test_tourism(self):
        # OECD -> Industry and Services -> Inbound Tourism
        result = read_jsdmx(os.path.join(self.dirpath, 'jsdmx',
                                         'tourism.json'))
        assert isinstance(result, pd.DataFrame)

        exp_col = pd.MultiIndex.from_product(
            [['Japan'], ['China', 'Hong Kong, China',
                         'Total international arrivals',
                         'Total international receipts',
                         'International passenger transport receipts',
                         'International travel receipts',
                         'Korea', 'Chinese Taipei', 'United States']],
            names=['Country', 'Variable'])
        exp_idx = pd.DatetimeIndex(['2004', '2005', '2006', '2007',
                                    '2008', '2009', '2010', '2011',
                                    '2012'], name='Year')

        values = np.array([
            [616, 300, 6138, 1550, 330, 1220, 1588, 1081, 760],
            [653, 299, 6728, 1710, 340, 1370, 1747, 1275, 822],
            [812, 352, 7334, 1330, 350, 980, 2117, 1309, 817],
            [942, 432, 8347, 1460, 360, 1100, 2601, 1385, 816],
            [1000, 550, 8351, 1430, 310, 1120, 2382, 1390, 768],
            [1006, 450, 6790, 1170, 210, 960, 1587, 1024, 700],
            [1413, 509, 8611, 1350, 190, 1160, 2440, 1268, 727],
            [1043, 365, 6219, 1000, 100, 900, 1658, 994, 566],
            [1430, 482, 8368, 1300, 100, 1200, 2044, 1467, 717]])
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(result, expected)

    def test_land_use(self):
        # OECD -> Environment -> Resources Land Use
        result = read_jsdmx(os.path.join(self.dirpath, 'jsdmx',
                                         'land_use.json'))
        assert isinstance(result, pd.DataFrame)
        result = result.loc['2010':'2011']

        exp_col = pd.MultiIndex.from_product([
            ['Japan', 'United States'],
            ['Arable land and permanent crops',
             'Arable and cropland % land area',
             'Total area', 'Forest', 'Forest % land area',
             'Land area', 'Permanent meadows and pastures',
             'Meadows and pastures % land area', 'Other areas',
             'Other % land area']], names=['Country', 'Variable'])
        exp_idx = pd.DatetimeIndex(['2010', '2011'], name='Year')
        values = np.array([[45930, 12.601, 377950, 249790, 68.529, 364500,
                            np.nan, np.nan, 68780, 18.87, 1624330, 17.757,
                            9831510, 3040220, 33.236, 9147420, 2485000,
                            27.166, 1997870, 21.841],
                           [45610, 12.513, 377955, 249878, 68.554, 364500,
                            np.nan, np.nan, 69012, 18.933, 1627625, 17.793,
                            9831510, 3044048, 33.278, 9147420, 2485000,
                            27.166, 1990747, 21.763]])
        expected = pd.DataFrame(values, index=exp_idx, columns=exp_col)
        tm.assert_frame_equal(result, expected)



__version__ = version = '0.5.0'

from .data import (get_components_yahoo, get_data_famafrench, get_data_google, get_data_yahoo, get_data_enigma,  # noqa
        get_data_yahoo_actions, get_quote_google, get_quote_yahoo, DataReader, Options)  # noqa
